from metagen.main import PTRMetagen
from metagen.metadata import Application, Configuration, Scope, Place, Period, LayerTemplate, Attribute, SpatialWMS, \
    RelationSpatial, SpatialAttribute, SpatialWMTS, SpatialVector, Style, Case, RelationAttribute, Tag
from metagen.config.config import Config

__version__ = '1.1.17'